<?php

namespace Litematrimony\Profile\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litematrimony\Profile\Events\VisitAction as VisitActionEvent;
use Litematrimony\Profile\Notifications\VisitAction as VisitActionNotification;
use Litepie\Actions\Concerns\AsAction;

class VisitAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the VisitActionEvent.
     *
     * @param   VisitActionEvent  $event
     * @return mixed
     */
    public function handle(VisitActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $visit.
     *
     * @param   VisitActionEvent  $event
     * @return void
     */
    public function create(VisitActionEvent $event)
    {
        $client = $event->$visit->client;
        Notification::send($client, new VisitActionNotification($event));
    }

    /**
     * Handle the VisitActionEvent as a listener.
     *
     * @param   VisitActionEvent  $event
     * @return mixed
     */
    public function asListener(VisitActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   VisitActionEvent  $event
     * @return bool
     */
    private function isAllowed(VisitActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
