<?php

namespace Litematrimony\Profile\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litematrimony\Profile\Events\VisitWorkflow as VisitWorkflowEvent;
use Litematrimony\Profile\Notifications\VisitWorkflow as VisitWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class VisitWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(VisitWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $visit is submitted.
     *
     * @param VisitWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(VisitWorkflowEvent $event)
    {
        $client = $event->$visit->client;
        Notification::send($client, new VisitWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $visit is published.
     *
     * @param VisitWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(VisitWorkflowEvent $event)
    {
        $client = $event->$visit->client;
        Notification::send($client, new VisitWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $visit is approved.
     *
     * @param VisitWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(VisitWorkflowEvent $event)
    {
        $client = $event->$visit->client;
        Notification::send($client, new VisitWorkflowNotification($event));
    }

    /**
     * Handles the $visit workflow event as a listener.
     *
     * @param VisitWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(VisitWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
