<?php
namespace Litematrimony\Profile\Notifications;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use Litematrimony\Profile\Events\VisitAction as VisitActionEvent;
use Litematrimony\Profile\Models\Visit;

class VisitAction extends Notification implements ShouldQueue
{
    use Queueable;

    protected $visit;

    protected $action;

    /**
     * Create a new notification instance.
     *
     * @param VisitActionEvent $visit
     * @param string $action
     */
    public function __construct(VisitActionEvent $visit, string $action)
    {
        $this->action = $action;
        $this->visit = $visit;
    }

    /**
     * Get the mail representation of the notification.
     *
     * @param mixed $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function toMail($notifiable)
    {
        return (new MailMessage)
            ->subject('Visit Approved - ' . config('app.name'))
            ->view('profile::visit.email.action', ['data' => $this->visit]);
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param object $notifiable
     * @return array<int, string>
     */
    public function via(object $notifiable): array
    {
        return ['mail'];
    }
}
